@@ -35,6 +35,7 @@
 #include <arpa/inet.h>
 #include <signal.h>
 #include <sys/signalfd.h>
+#include <errno.h>
 
 #include <glib.h>
 #include <utmp.h>
@@ -44,10 +45,12 @@
 #include <sys/stat.h>
 
 #include "gatserver.h"
+#include "gatppp.h"
 #include "ringbuffer.h"
 
 #define DEFAULT_TCP_PORT 12346
 #define DEFAULT_SOCK_PATH "./server_sock"
+#define IFCONFIG_PATH "/sbin/ifconfig"
 
 static int modem_mode = 0;
 static int modem_creg = 0;
@@ -61,13 +64,19 @@ struct sock_server{
 
 static GMainLoop *mainloop;
 static GAtServer *server;
+static GAtPPP *ppp;
 unsigned int server_watch;
 
-static gboolean server_cleanup()
+static gboolean server_cleanup(void)
 {
 	if (server_watch)
 		g_source_remove(server_watch);
 
+	if (ppp) {
+		g_at_ppp_unref(ppp);
+		ppp = NULL;
+	}
+
 	g_at_server_unref(server);
 	server = NULL;
 
@@ -83,10 +92,110 @@ static void server_debug(const char *str, void *data)
 	g_print("%s: %s\n", (char *) data, str);
 }
 
-static void cgmi_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static gboolean execute(const char *cmd)
+{
+	int status;
+
+	status = system(cmd);
+	if (status < 0) {
+		g_print("Failed to execute command: %s\n", strerror(errno));
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static void ppp_connect(const char *iface, const char *local, const char *peer,
+			const char *dns1, const char *dns2,
+			gpointer user)
+{
+	char buf[512];
+
+	g_print("Network Device: %s\n", iface);
+	g_print("IP Address: %s\n", local);
+	g_print("Peer IP Address: %s\n", peer);
+	g_print("Primary DNS Server: %s\n", dns1);
+	g_print("Secondary DNS Server: %s\n", dns2);
+
+	snprintf(buf, sizeof(buf), "%s %s up", IFCONFIG_PATH, iface);
+	execute(buf);
+
+	snprintf(buf, sizeof(buf), "%s %s %s pointopoint %s", IFCONFIG_PATH,
+				iface, local, peer);
+	execute(buf);
+
+	snprintf(buf, sizeof(buf), "echo 1 > /proc/sys/net/ipv4/ip_forward");
+	execute(buf);
+}
+
+static void ppp_disconnect(GAtPPPDisconnectReason reason, gpointer user)
 {
 	GAtServer *server = user;
 
+	g_print("PPP Link down: %d\n", reason);
+
+	g_at_ppp_unref(ppp);
+	ppp = NULL;
+
+	if (reason == G_AT_PPP_REASON_LINK_DEAD) {
+		g_at_server_unref(server);
+		server = NULL;
+		return;
+	}
+
+	g_at_server_resume(server);
+	g_at_server_set_debug(server, server_debug, "Server");
+
+	g_at_server_send_final(server, G_AT_SERVER_RESULT_NO_CARRIER);
+}
+
+static gboolean update_ppp(gpointer user)
+{
+	GAtPPP *ppp = user;
+
+	g_at_ppp_set_server_info(ppp, "192.168.1.2",
+					"10.10.10.10", "10.10.10.11");
+
+	return FALSE;
+}
+
+static gboolean setup_ppp(gpointer user)
+{
+	GAtServer *server = user;
+	GAtIO *io;
+
+	if (getuid() != 0) {
+		g_print("Need root privilege for PPP connection\n");
+		return FALSE;
+	}
+
+	io = g_at_server_get_io(server);
+
+	g_at_server_suspend(server);
+
+	/* open ppp */
+	ppp = g_at_ppp_server_new_from_io(io, "192.168.1.1");
+	if (ppp == NULL) {
+		g_at_server_resume(server);
+		return FALSE;
+	}
+
+	g_at_ppp_set_debug(ppp, server_debug, "PPP");
+
+	g_at_ppp_set_credentials(ppp, "", "");
+
+	/* set connect and disconnect callbacks */
+	g_at_ppp_set_connect_function(ppp, ppp_connect, server);
+	g_at_ppp_set_disconnect_function(ppp, ppp_disconnect, server);
+
+	g_idle_add(update_ppp, ppp);
+
+	return FALSE;
+}
+
+static void cgmi_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
+{
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_COMMAND_ONLY:
 		g_at_server_send_info(server, "oFono", TRUE);
@@ -100,10 +209,9 @@ static void cgmi_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	};
 }
 
-static void cgmm_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cgmm_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
-
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_COMMAND_ONLY:
 		g_at_server_send_info(server, "oFono pre-1.0", TRUE);
@@ -117,9 +225,9 @@ static void cgmm_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	};
 }
 
-static void cgmr_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cgmr_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
 	char buf[256];
 
 	switch (type) {
@@ -136,10 +244,9 @@ static void cgmr_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	};
 }
 
-static void cgsn_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cgsn_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
-
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_COMMAND_ONLY:
 		g_at_server_send_info(server, "123456789", TRUE);
@@ -162,9 +269,9 @@ static gboolean send_ok(gpointer user)
 	return FALSE;
 }
 
-static void cfun_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cfun_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
 	char buf[12];
 
 	switch (type) {
@@ -183,7 +290,7 @@ static void cfun_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 		int mode;
 
 		g_at_result_iter_init(&iter, cmd);
-		g_at_result_iter_next(&iter, "+CFUN=");
+		g_at_result_iter_next(&iter, "");
 
 		if (g_at_result_iter_next_number(&iter, &mode) == FALSE)
 			goto error;
@@ -210,10 +317,9 @@ error:
 	g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
 }
 
-static void cpin_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cpin_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
-
 	if (modem_mode == 0) {
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
 		return;
@@ -255,10 +361,9 @@ static gboolean do_netreg(gpointer user)
 	return FALSE;
 }
 
-static void cops_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cops_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
-
 	if (modem_mode == 0) {
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
 		return;
@@ -279,7 +384,7 @@ static void cops_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 		int mode;
 
 		g_at_result_iter_init(&iter, cmd);
-		g_at_result_iter_next(&iter, "+COPS=");
+		g_at_result_iter_next(&iter, "");
 
 		if (g_at_result_iter_next_number(&iter, &mode) == TRUE) {
 			if (mode == 0) {
@@ -299,9 +404,9 @@ static void cops_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
-static void creg_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void creg_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
 	char buf[20];
 
 	if (modem_mode == 0) {
@@ -326,7 +431,7 @@ static void creg_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 		int mode;
 
 		g_at_result_iter_init(&iter, cmd);
-		g_at_result_iter_next(&iter, "+CREG=");
+		g_at_result_iter_next(&iter, "");
 
 		if (g_at_result_iter_next_number(&iter, &mode) == FALSE)
 			goto error;
@@ -348,9 +453,9 @@ error:
 	g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
 }
 
-static void cgreg_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cgreg_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
 	char buf[20];
 
 	if (modem_mode == 0) {
@@ -375,7 +480,7 @@ static void cgreg_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 		int mode;
 
 		g_at_result_iter_init(&iter, cmd);
-		g_at_result_iter_next(&iter, "+CGREG=");
+		g_at_result_iter_next(&iter, "");
 
 		if (g_at_result_iter_next_number(&iter, &mode) == FALSE)
 			goto error;
@@ -397,9 +502,9 @@ error:
 	g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
 }
 
-static void cgatt_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cgatt_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
 	char buf[12];
 
 	if (modem_mode == 0) {
@@ -423,7 +528,7 @@ static void cgatt_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 		int mode;
 
 		g_at_result_iter_init(&iter, cmd);
-		g_at_result_iter_next(&iter, "+CGATT=");
+		g_at_result_iter_next(&iter, "");
 
 		if (g_at_result_iter_next_number(&iter, &mode) == FALSE)
 			goto error;
@@ -450,10 +555,9 @@ error:
 	g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
 }
 
-static void cgdata_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cgdata_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
-
 	if (modem_mode == 0) {
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
 		return;
@@ -467,7 +571,8 @@ static void cgdata_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
 		break;
 	case G_AT_SERVER_REQUEST_TYPE_SET:
-		g_at_server_send_final(server, G_AT_SERVER_RESULT_CONNECT);
+		g_at_server_send_intermediate(server, "CONNECT");
+		g_idle_add(setup_ppp, server);
 		break;
 	default:
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
@@ -475,10 +580,9 @@ static void cgdata_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
-static void cgdcont_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cgdcont_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
-
 	if (modem_mode == 0) {
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
 		return;
@@ -500,10 +604,9 @@ static void cgdcont_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
-static void cimi_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cimi_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
-
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_COMMAND_ONLY:
 		g_at_server_send_info(server, "246813579", TRUE);
@@ -518,10 +621,9 @@ static void cimi_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
-static void csms_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void csms_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
-
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
@@ -540,10 +642,9 @@ static void csms_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
-static void cmgf_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cmgf_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
-
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
@@ -562,9 +663,9 @@ static void cmgf_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
-static void cpms_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cpms_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
 	char buf[2048];
 
 	switch (type) {
@@ -585,9 +686,9 @@ static void cpms_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
-static void cnmi_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cnmi_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
 	char buf[2048];
 
 	switch (type) {
@@ -608,9 +709,9 @@ static void cnmi_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
-static void cscs_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cscs_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
 	char buf[2048];
 
 	switch (type) {
@@ -631,10 +732,9 @@ static void cscs_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
-static void cmgl_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cmgl_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
-
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
@@ -648,9 +748,9 @@ static void cmgl_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
-static void cpbs_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
+static void cpbs_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
 {
-	GAtServer *server = user;
 	char buf[2048];
 
 	switch (type) {
@@ -671,30 +771,63 @@ static void cpbs_cb(GAtServerRequestType type, GAtResult *cmd, gpointer user)
 	}
 }
 
+static void dial_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *cmd, gpointer user)
+{
+	GAtServerResult res = G_AT_SERVER_RESULT_ERROR;
+	GAtResultIter iter;
+	const char *dial_str;
+	char c;
+
+	if (type != G_AT_SERVER_REQUEST_TYPE_SET)
+		goto error;
+
+	g_at_result_iter_init(&iter, cmd);
+	g_at_result_iter_next(&iter, "");
+
+	dial_str = g_at_result_iter_raw_line(&iter);
+	if (dial_str == NULL)
+		goto error;
+
+	g_print("dial call %s\n", dial_str);
+
+	c = *dial_str;
+	if (c == '*' || c == '#' || c == 'T' || c == 't') {
+		g_at_server_send_intermediate(server, "CONNECT");
+		g_idle_add(setup_ppp, server);
+	}
+
+	return;
+
+error:
+	g_at_server_send_final(server, res);
+}
+
 static void add_handler(GAtServer *server)
 {
 	g_at_server_set_debug(server, server_debug, "Server");
 
-	g_at_server_register(server, "+CGMI",    cgmi_cb,    server, NULL);
-	g_at_server_register(server, "+CGMM",    cgmm_cb,    server, NULL);
-	g_at_server_register(server, "+CGMR",    cgmr_cb,    server, NULL);
-	g_at_server_register(server, "+CGSN",    cgsn_cb,    server, NULL);
-	g_at_server_register(server, "+CFUN",    cfun_cb,    server, NULL);
-	g_at_server_register(server, "+CPIN",    cpin_cb,    server, NULL);
-	g_at_server_register(server, "+COPS",    cops_cb,    server, NULL);
-	g_at_server_register(server, "+CREG",    creg_cb,    server, NULL);
-	g_at_server_register(server, "+CGREG",   cgreg_cb,   server, NULL);
-	g_at_server_register(server, "+CGATT",   cgatt_cb,   server, NULL);
-	g_at_server_register(server, "+CGDATA",  cgdata_cb,  server, NULL);
-	g_at_server_register(server, "+CGDCONT", cgdcont_cb, server, NULL);
-	g_at_server_register(server, "+CIMI",    cimi_cb,    server, NULL);
-	g_at_server_register(server, "+CSMS",    csms_cb,    server, NULL);
-	g_at_server_register(server, "+CMGF",    cmgf_cb,    server, NULL);
-	g_at_server_register(server, "+CPMS",    cpms_cb,    server, NULL);
-	g_at_server_register(server, "+CNMI",    cnmi_cb,    server, NULL);
-	g_at_server_register(server, "+CSCS",    cscs_cb,    server, NULL);
-	g_at_server_register(server, "+CMGL",    cmgl_cb,    server, NULL);
-	g_at_server_register(server, "+CPBS",    cpbs_cb,    server, NULL);
+	g_at_server_register(server, "+CGMI",    cgmi_cb,    NULL, NULL);
+	g_at_server_register(server, "+CGMM",    cgmm_cb,    NULL, NULL);
+	g_at_server_register(server, "+CGMR",    cgmr_cb,    NULL, NULL);
+	g_at_server_register(server, "+CGSN",    cgsn_cb,    NULL, NULL);
+	g_at_server_register(server, "+CFUN",    cfun_cb,    NULL, NULL);
+	g_at_server_register(server, "+CPIN",    cpin_cb,    NULL, NULL);
+	g_at_server_register(server, "+COPS",    cops_cb,    NULL, NULL);
+	g_at_server_register(server, "+CREG",    creg_cb,    NULL, NULL);
+	g_at_server_register(server, "+CGREG",   cgreg_cb,   NULL, NULL);
+	g_at_server_register(server, "+CGATT",   cgatt_cb,   NULL, NULL);
+	g_at_server_register(server, "+CGDATA",  cgdata_cb,  NULL, NULL);
+	g_at_server_register(server, "+CGDCONT", cgdcont_cb, NULL, NULL);
+	g_at_server_register(server, "+CIMI",    cimi_cb,    NULL, NULL);
+	g_at_server_register(server, "+CSMS",    csms_cb,    NULL, NULL);
+	g_at_server_register(server, "+CMGF",    cmgf_cb,    NULL, NULL);
+	g_at_server_register(server, "+CPMS",    cpms_cb,    NULL, NULL);
+	g_at_server_register(server, "+CNMI",    cnmi_cb,    NULL, NULL);
+	g_at_server_register(server, "+CSCS",    cscs_cb,    NULL, NULL);
+	g_at_server_register(server, "+CMGL",    cmgl_cb,    NULL, NULL);
+	g_at_server_register(server, "+CPBS",    cpbs_cb,    NULL, NULL);
+	g_at_server_register(server, "D",        dial_cb,    NULL, NULL);
 }
 
 static void server_destroy(gpointer user)
@@ -706,15 +839,13 @@ static void server_destroy(gpointer user)
 
 static void set_raw_mode(int fd)
 {
-	struct termios options;
+	struct termios ti;
 
-	tcgetattr(fd, &options);
-
-	/* Set TTY as raw mode to disable echo back of input characters
-	 * when they are received from Modem to avoid feedback loop */
-	options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
-
-	tcsetattr(fd, TCSANOW, &options);
+	memset(&ti, 0, sizeof(ti));
+	tcgetattr(fd, &ti);
+	tcflush(fd, TCIOFLUSH);
+	cfmakeraw(&ti);
+	tcsetattr(fd, TCSANOW, &ti);
 }
 
 static gboolean create_tty(const char *modem_path)
@@ -722,9 +853,8 @@ static gboolean create_tty(const char *modem_path)
 	int master, slave;
 	char pty_name[256];
 	GIOChannel *server_io;
-	GIOChannel *client_io;
 
-	if (!modem_path)
+	if (modem_path == NULL)
 		return FALSE;
 
 	if (openpty(&master, &slave, pty_name, NULL, NULL) < 0)
@@ -732,21 +862,20 @@ static gboolean create_tty(const char *modem_path)
 
 	set_raw_mode(slave);
 
-	client_io = g_io_channel_unix_new(slave);
-	g_io_channel_set_close_on_unref(client_io, TRUE);
-
 	g_print("new pty is created at %s\n", pty_name);
 
 	server_io = g_io_channel_unix_new(master);
 
 	server = g_at_server_new(server_io);
-	if (!server) {
+	if (server == NULL) {
 		g_io_channel_shutdown(server_io, FALSE, NULL);
 		g_io_channel_unref(server_io);
 
 		return FALSE;
 	}
 
+	g_io_channel_unref(server_io);
+
 	return TRUE;
 }
 
@@ -771,7 +900,7 @@ static gboolean on_socket_connected(GIOChannel *chan, GIOCondition cond,
 	server = g_at_server_new(client_io);
 	g_io_channel_unref(client_io);
 
-	if (!server)
+	if (server == NULL)
 		goto error;
 
 	add_handler(server);
@@ -810,7 +939,7 @@ static struct sock_server *socket_common(int sk, struct sockaddr *addr,
 	}
 
 	sock = g_try_new0(struct sock_server, 1);
-	if (!sock)
+	if (sock == NULL)
 		return FALSE;
 
 	sock->server_sock = sk;
@@ -825,7 +954,7 @@ static gboolean create_tcp(const char *modem_path, int port)
 	struct sock_server *server;
 	GIOChannel *server_io;
 
-	if (!modem_path)
+	if (modem_path == NULL)
 		return FALSE;
 
 	sk = socket(PF_INET, SOCK_STREAM, 0);
@@ -842,7 +971,7 @@ static gboolean create_tcp(const char *modem_path, int port)
 	addr.sin_port = htons(port);
 
 	server = socket_common(sk, (struct sockaddr *) &addr, modem_path);
-	if (!server)
+	if (server == NULL)
 		return FALSE;
 
 	g_print("new tcp is created at tcp port %d\n", port);
@@ -868,7 +997,7 @@ static gboolean create_unix(const char *modem_path, const char *sock_path)
 	struct sock_server *server;
 	GIOChannel *server_io;
 
-	if (!modem_path)
+	if (modem_path == NULL)
 		return FALSE;
 
 	sk = socket(AF_UNIX, SOCK_STREAM, 0);
@@ -888,7 +1017,7 @@ static gboolean create_unix(const char *modem_path, const char *sock_path)
 	unlink(addr.sun_path);
 
 	server = socket_common(sk, (struct sockaddr *) &addr, modem_path);
-	if (!server)
+	if (server == NULL)
 		return FALSE;
 
 	g_print("new unix socket is created at %s\n", sock_path);
@@ -954,7 +1083,7 @@ static gboolean signal_cb(GIOChannel *channel, GIOCondition cond, gpointer data)
 	return TRUE;
 }
 
-static int create_signal_io()
+static int create_signal_io(void)
 {
 	sigset_t mask;
 	GIOChannel *signal_io;
